package v3

import (
	"encoding/json"
	"htwkalender/service/functions"
	"log/slog"
	"net/http"
)

func parseEvents(url string, client *http.Client) (Events, error) {

	// the url is paginated, so we need to fetch all pages
	// example url: https://luna.htwk-leipzig.de/api/veranstaltungen?page=1&itemsPerPage=100
	// the itemsPerPage is set to 100, so we need to fetch all pages until we get an empty response

	var fetchedEvents Events
	var itemsPerPage = 999

	responses, err := paginatedFetch(url, itemsPerPage, client)

	if err != nil {
		slog.Error("Error while fetching events", err)
		return Events{}, err
	}

	for _, response := range responses {
		var events Events
		err = json.Unmarshal([]byte(response), &events)
		if err != nil {
			slog.Error("Error while unmarshalling events", err)
			return Events{}, err
		}

		// cut api iri prefix
		for i, event := range events.Events {

			events.Events[i].EventType = functions.RemoveIriPrefix(event.EventType, 32)
			events.Events[i].Faculty = functions.RemoveIriPrefix(event.Faculty, 32)
			events.Events[i].Module = functions.RemoveIriPrefix(event.Module, 32)
			events.Events[i].Semester = functions.RemoveIriPrefix(event.Semester, 2)
			for j, professors := range event.Professors {
				events.Events[i].Professors[j] = functions.RemoveIriPrefix(professors, 32)
			}
			for j, rooms := range event.Rooms {
				events.Events[i].Rooms[j] = functions.RemoveIriPrefix(rooms, 32)
			}
			for j, courses := range event.Courses {
				events.Events[i].Courses[j] = functions.RemoveIriPrefix(courses, 32)
			}
		}

		fetchedEvents.Events = append(fetchedEvents.Events, events.Events...)
		fetchedEvents.TotalItems = events.TotalItems
	}

	return fetchedEvents, nil
}
